/*
 *    Program to calibrate coupler arms of an RF power measurement.  It uses power
 *    meters and an RF source.  Part of e-factory
 *    Programmer:	Danny Holstein
 *    Started:		2/21/97
 */

#include "forms.h"
#include <stdlib.h>
#include "e_factory.h"
#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <ib.h>
#include "drivers.h"

#define USE_SRQ 1
#define READS 5
#define WAIT 1
#define MAX_DRIVE_PTS 30
#define MAX_RES 1024

extern int ibsta,ibcnt,iberr;
char cmd[130];
char res[MAX_RES], message[1024];

FD_calibrate *fd_calibrate;
int preemptive_handler(FL_OBJECT *ob, int event,
                          FL_Coord mx, FL_Coord my, int key, void *xev);

float freq[MAX_FREQ_PTS], in_cal[MAX_FREQ_PTS], out_cal[MAX_FREQ_PTS][2];
float in_po[MAX_FREQ_PTS][MAX_DRIVE_PTS], out_po[MAX_FREQ_PTS][MAX_DRIVE_PTS][2];
float rf_drive_lo, rf_drive_hi, drive_pwr, set_power, start_sweep, stop_sweep;
float del_freq, f_start, f_stop;
int max_freq;
float mtr_range_drive[MAX_FREQ_PTS][5], mtr_range_op[6];
int mtr_min_range[MAX_FREQ_PTS], drive_levels[MAX_FREQ_PTS];

float in_cplr_freq[MAX_FREQ_PTS], out_cplr_freq[MAX_FREQ_PTS];
float in_cplr[MAX_FREQ_PTS], out_cplr[MAX_FREQ_PTS];
int in_cplr_pts, out_cplr_pts;

void save_cal(int file);
int read_cplr_factors();

int transfer_curve()
{
   int i, j, n, n_freq, last_output_range, min_range;
   int sweeper, in_meter, out_meter;
   int sweeper_model, in_meter_model, out_meter_model, mtr_level;
   float probable_pwr;
   float cplr_del_freq, calibration;
   SWEEPER_PAR *parameters;
/*
 *    start button has been pressed, load input values
 */
  f_start = atof(fl_get_input(fd_calibrate->CAL_start_freq));
  f_stop = atof(fl_get_input(fd_calibrate->CAL_stop_freq));
  max_freq =  fl_get_counter_value(fd_calibrate->CAL_freqs);
  min_range =  fl_get_counter_value(fd_calibrate->CAL_range);
  if (read_cplr_factors() != 0) return(0); 
  if (in_cplr_freq[0] > f_start || out_cplr_freq[0] > f_start) {
    fl_show_message("Start frequency below coupler minimum",
                    "Re-enter data","");
    return(0);}
  if (in_cplr_freq[in_cplr_pts-1] < f_stop || out_cplr_freq[out_cplr_pts-1] < f_stop) {
    fl_show_message("Stop frequency above coupler maximum",
                    "Re-enter data","");
    return(0);}
/*
 *   initialize basic variables
 */
  del_freq = 0.;
  if (max_freq > 1) {del_freq = (f_stop - f_start)/(max_freq - 1);}
  for(n_freq=0; n_freq<max_freq; n_freq++) {freq[n_freq] = f_start + n_freq*del_freq;}
  if (del_freq<0. || f_start==0. ) {
     return(0);}
/*
 *   initialize sweeper, check band and turn off RF for power meter zeroing
 */
  if((sweeper = ibfind("sweeper")) & ERR){
    gpiberr("ibfind err");
    exit(1);
  }
  if(ibclr(sweeper) & ERR){
    gpiberr("clr err");
    exit(1);
  }
  
  sweeper_model = atoi(parse("sweeper","type"));
  parameters = sweeper_init(sweeper, sweeper_model);
  rf_drive_lo = parameters->rf_drive_lo;
  rf_drive_hi = parameters->rf_drive_hi;
  start_sweep = parameters->start_sweep;
  stop_sweep = parameters->stop_sweep;
  mtr_level = parameters->mtr_level;
   
  if (start_sweep > f_start) {
    fl_show_message("Start frequency below sweeper minimum",
                    "Re-enter data","");
    return(0);}
  if (stop_sweep < f_stop) {
    fl_show_message("Stop frequency above sweeper maximum",
                    "Re-enter data","");
    return(0);}

  sweeper_rf_off(sweeper, sweeper_model);    
  if (fl_check_forms() == fd_calibrate->CAL_abort) return(0);
/*
 *   initialize and zero power meters
 */
  in_meter_model = atoi(parse("in_meter","type"));
  if((in_meter = ibfind("in_meter")) & ERR){
    gpiberr("ibfind err");
    exit(1);
  }
  if( ibtmo(in_meter,15) & ERR ){
    gpiberr("tmo err");
    exit(1);
  }
  mtr_zero(in_meter,in_meter_model);
  
  out_meter_model = atoi(parse("out_meter","type"));
  if((out_meter = ibfind("out_meter")) & ERR){
    gpiberr("ibfind err");
    exit(1);
  }
  if( ibtmo(out_meter,15) & ERR){
    gpiberr("tmo err");
    exit(1);
  }
  mtr_zero(out_meter,out_meter_model);
/*
 *   read status and check to see if meters have been zeroed
 */
  for (i=1; i<30; i++) {
    sleep(1);
    if (mtr_zeroed(in_meter,in_meter_model) == 1 &&
        mtr_zeroed(out_meter,out_meter_model) == 1) break;
    if (i>=25) exit(1);
  }
  if (fl_check_forms() == fd_calibrate->CAL_abort) return (0);
  
  for (i = 1; i <= 5; i++) {mtr_range_op[i] = -1000.;}
/*
 ******  Read Transfer Curves, main loop ***************************
 */
 
  for(n_freq=0; n_freq<max_freq; n_freq++) {
  
    if (fl_check_forms() == fd_calibrate->CAL_abort) return (0);
/*
 *   record power ranges for frequency
 */
    set_sweeper_on(sweeper,freq[n_freq],rf_drive_lo,sweeper_model);
  
    i = min_range;
    mtr_min_range[n_freq] = 5;
    if (mtr_level) {   /* The program will use meter range switching to extend the dynamic range */
      while (i<6) {
	mtr_range_drive[n_freq][i] = pwr_set_range(in_meter,i,in_meter_model);
	i++ ;
      }
      for (i=min_range; i<6; i++) {
	if ( mtr_range_drive[n_freq][i] != -1000.0 &
            (mtr_range_drive[n_freq][i+1] 
                       - mtr_range_drive[n_freq][i]) > 3.0 ) {
          if ( i < mtr_min_range[n_freq]) {mtr_min_range[n_freq] = i;}
          break;
	}
      }
    }
    else {   /* The program will use the sweeper dynamic range */
      if (rf_drive_lo - 4 < parameters->rf_drive_lo ) {set_power = parameters->rf_drive_lo;}
      else {set_power = rf_drive_lo - 4;}
      while (set_power<rf_drive_hi) {
        set_rf_power(sweeper,set_power,sweeper_model);
	mtr_range_drive[n_freq][i] = pwr_set_range(in_meter,i,in_meter_model);
	if (mtr_range_drive[n_freq][i] != -1000.0) {
	  rf_drive_lo = set_power + 2;
	  mtr_min_range[n_freq] = min_range;
          for (i=min_range+1; i<6; i++) {mtr_range_drive[n_freq][i] = 100.0;}
	  break; }
	set_power = set_power + 2.0 ;
      }
      if (mtr_range_drive[n_freq][i] == -1000.0) {
	  printf("no drive measured");
	  return (0); }
    }
  
/*
 *  "temporary" work-around  
 */
           if((out_meter = ibfind("out_meter")) & ERR){
             gpiberr("ibfind err");                    
             exit(1);
           }
    printf("frequency = %f\n",freq[n_freq]);
/*
 *   record transfer curve for frequency
 */
   n = 0; drive_levels[n_freq] = n;
   last_output_range = 1;
   for (drive_pwr=mtr_range_drive[n_freq][mtr_min_range[n_freq]];
        drive_pwr<20;  drive_pwr=drive_pwr+2) {
     for (i = 5; i >= mtr_min_range[n_freq]; i--) {
       if (drive_pwr >= mtr_range_drive[n_freq][i]) {
         set_power = drive_pwr - mtr_range_drive[n_freq][i] + rf_drive_lo ;
         if (set_power < rf_drive_lo ) {set_power = rf_drive_lo ;}
         if (set_power > rf_drive_hi) break;
         set_rf_power(sweeper,set_power,sweeper_model);
       
         if (mtr_level) {set_range_rd(in_meter, i,in_meter_model);}
	 else {if (n==0) {set_range_rd(in_meter, i,in_meter_model);}
	         else{set_range_rd(in_meter, read_range(in_meter, in_meter_model),in_meter_model);}}
         if (n == 0) {set_range_rd(out_meter, 0, out_meter_model);}
         else {
           probable_pwr = out_po[n_freq][n-1][0] + drive_pwr - in_po[n_freq][n-1]; j = 0;
           for (i = 1; i < 5; i++) {
             if (mtr_range_op[i] != -1000.0 &&
                 probable_pwr > mtr_range_op[i] &&
                 mtr_range_op[i+1] != -1000.0 &&
                 probable_pwr < mtr_range_op[i+1]) { j = i; break;}
           }
           if (mtr_range_op[5] != -1000.0 &&
               probable_pwr > mtr_range_op[5]) { j = 5;}
           set_range_rd(out_meter, j, out_meter_model);
         }
       
         n++; drive_levels[n_freq] = n;
       
         in_po[n_freq][n] = read_power(in_meter, in_meter_model);
         if (in_po[n_freq][n] == -1000.0) {
           fl_show_message("Input meter read error","Check connections","");
           return (0);
         }
       
         out_po[n_freq][n][0] = read_power(out_meter, out_meter_model);
         if (out_po[n_freq][n][0] == -1000.0) {
           printf("under range");
         }
       
         j = read_range(out_meter,out_meter_model);
         if (j > last_output_range ) {
           mtr_range_op[j] = out_po[n_freq][n][0];
           last_output_range = j;
         }
         printf("%d, %f, %f, %d\n",n, in_po[n_freq][n], out_po[n_freq][n][0],n_freq);
         break;
       }
     }
     if (set_power > rf_drive_hi &
                  (i >= 5 || !mtr_level)) break ;     /* input maximum */
     if (n > 1) {                                      /* test for output saturation */
       if (out_po[n_freq][n][0] <=
           out_po[n_freq][n-1][0] - .2) break ;        /* test for input limits */
       if (in_po[n_freq][n] <=
           in_po[n_freq][n-1] + .2) break ;
     }
   }
 }
/*
 ******  Read Transfer Curves, end loop ***************************
 */
 
/*
 *   add coupler factors
 */
  cplr_del_freq = out_cplr_freq[1] - out_cplr_freq[0];
  for(i=0; i<max_freq; i++) {
    if (freq[i] == out_cplr_freq[out_cplr_pts-1]) {
      calibration = -10*log10(out_cplr[out_cplr_pts-1]); }
    else {
      n = (freq[i] - out_cplr_freq[0])/cplr_del_freq;
      calibration = -10*log10((out_cplr_freq[n+1] - freq[i])/cplr_del_freq * out_cplr[n] 
                              + (freq[i] - out_cplr_freq[n])/cplr_del_freq * out_cplr[n+1]);
    }
    
    for(j=1; j<=drive_levels[i]; j++) {
      out_po[i][j][0] += calibration;
    }
  }
  
  cplr_del_freq = in_cplr_freq[1] - in_cplr_freq[0];
  for(i=0; i<max_freq; i++) {
    if (freq[i] == in_cplr_freq[in_cplr_pts-1]) {
      calibration = -10*log10(in_cplr[in_cplr_pts-1]); }
    else {
      n = (freq[i] - in_cplr_freq[0])/cplr_del_freq;
      calibration = -10*log10((in_cplr_freq[n+1] - freq[i])/cplr_del_freq * in_cplr[n] 
                              + (freq[i] - in_cplr_freq[n])/cplr_del_freq * in_cplr[n+1]);
    }
    
    for(j=1; j<=drive_levels[i]; j++) {
      in_po[i][j] += calibration;
    }
  }

  sweeper_rf_off(sweeper, sweeper_model);
  return (1);
}

FD_calibrate *create_form_calibrate(void)
{
  FL_OBJECT *obj;
  FD_calibrate *fdui = (FD_calibrate *) fl_calloc(1, sizeof(*fdui));
  int i;

  fdui->calibrate = fl_bgn_form(FL_NO_BOX, 380, 320);
  obj = fl_add_box(FL_UP_BOX,0,0,380,320,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,120,180,120,20,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,10,50,230,120,"");
  obj = fl_add_box(FL_UP_BOX,0,0,380,40,"");
  fdui->CAL_file_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,10,10,60,20,"File");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_menu(obj, "Save Input Calibration|Save Output Calibration|Exit");
    for(i=1; i<=2; i++) {fl_set_menu_item_mode(obj, i, FL_PUP_GRAY);}
  fdui->CAL_display_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,90,10,60,20,"Display");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,CAL_display_menu_cb,0);
  fdui->CAL_help_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,310,10,60,20,"Help");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,CAL_help_menu_cb,0);
  fdui->CAL_start_freq = obj = fl_add_input(FL_FLOAT_INPUT,100,60,80,30,"Start Frequency");
  fdui->CAL_stop_freq = obj = fl_add_input(FL_FLOAT_INPUT,100,90,80,30,"Stop Frequency");
  fdui->CAL_start_ip = obj = fl_add_button(FL_NORMAL_BUTTON,260,60,110,30,"Calibrate Input");
    fl_set_object_color(obj,FL_SPRINGGREEN,FL_PALEGREEN);
    fl_set_object_lcol(obj,FL_DARKTOMATO);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,CAL_start_ip_cb,0);
  fdui->CAL_abort = obj = fl_add_button(FL_NORMAL_BUTTON,260,140,110,30,"Abort");
    fl_set_object_color(obj,FL_RED,FL_DARKTOMATO);
    fl_set_object_lcol(obj,FL_YELLOW);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
  obj = fl_add_frame(FL_ENGRAVED_FRAME,10,210,360,80,"");
  fdui->CAL_in_cal = obj = fl_add_input(FL_NORMAL_INPUT,120,220,220,30,"Input Calibration File");
    fl_set_object_prehandler(obj, preemptive_handler);
  fdui->CAL_out_cal = obj = fl_add_input(FL_NORMAL_INPUT,120,250,220,30,"Output Calibration File");
    fl_set_object_prehandler(obj, preemptive_handler);
  fdui->CAL_freqs = obj = fl_add_counter(FL_NORMAL_COUNTER,70,130,160,30,"Frequencies");
    fl_set_counter_bounds(obj,1.0,MAX_FREQ_PTS);
    fl_set_counter_step(obj,1.0,10.0);
    fl_set_counter_precision(obj,0);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
  fdui->CAL_range = obj = fl_add_counter(FL_SIMPLE_COUNTER,300,180,70,20,"Min. Range");
    fl_set_counter_bounds(obj,1.0,5);
    fl_set_counter_step(obj,1.0,1.0);
    fl_set_counter_precision(obj,0);
    fl_set_counter_value(obj, 5.0);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
  fdui->CAL_start_op = obj = fl_add_button(FL_NORMAL_BUTTON,260,100,110,30,"Calibrate Output");
    fl_set_object_color(obj,FL_SPRINGGREEN,FL_PALEGREEN);
    fl_set_object_lcol(obj,FL_DARKTOMATO);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,CAL_start_op_cb,0);
  fdui->CAL_2_path = obj = fl_add_checkbutton(FL_PUSH_BUTTON,120,180,80,20,"2 path (filters)");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,240,180,130,20,"");
  fl_end_form();

  return fdui;
}
/*---------------------------------------*/

void CAL_display_menu_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void CAL_help_menu_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

int CAL_check_exit()
{
   switch (fl_get_menu(fd_calibrate->CAL_file_menu)) {
      case 1:
         save_cal(1);
         return 0;
         break;
      case 2:
         save_cal(2);
         return 0;
         break;
      case 3:
         return 1;
         break;
      default:
         return 0;
         break;
   }
}

/*
 *   start an input circuit calibration
 */
void CAL_start_ip_cb(FL_OBJECT *ob, long data)
{
  int i, j, n_freq;
  float sum_coupler;
  if(transfer_curve()){
    for(n_freq=0; n_freq<max_freq; n_freq++) {
      sum_coupler = 0.; j = 0;
      for(i=2; i<drive_levels[n_freq] ; i++) {
	if(pow((out_po[n_freq][i+1][0]-out_po[n_freq][i-1][0]) - 
               ( in_po[n_freq][i+1]   -in_po[n_freq][i-1]),2.) < .10) {
          sum_coupler += out_po[n_freq][i][0] - in_po[n_freq][i];
          j++;
	}
      }
      in_cal[n_freq] = sum_coupler/j;
      printf("Cal = %f, j = %d\n", in_cal[n_freq], n_freq);
      fl_set_menu_item_mode(fd_calibrate->CAL_file_menu, 1, FL_PUP_NONE);
    }
  }
}

/*
 *   start an output circuit calibration, needs input calibration first
 */
void CAL_start_op_cb(FL_OBJECT *ob, long data)
{
  int i, j, n_freq;
  float sum_coupler;
  int switch_driver, switch_driver_model;
  
  if (fl_get_button(fd_calibrate->CAL_2_path)) {
    if((switch_driver = ibfind("switch_driver")) & ERR){
      gpiberr("ibfind err");
      exit(1);
    }
    switch_driver_model = atoi(parse("switch_driver","type"));
    switch_path_a(switch_driver, switch_driver_model);
  }
  if(transfer_curve()){
    for(n_freq=0; n_freq<max_freq; n_freq++) {
      for(i=1; i<drive_levels[n_freq]+1 ; i++) {
	in_po[n_freq][i] += in_cal[n_freq];
      }
    }

    for(n_freq=0; n_freq<max_freq; n_freq++) {
      sum_coupler = 0.; j = 0;
      for(i=2; i<drive_levels[n_freq] ; i++) {
	if(pow((out_po[n_freq][i+1][0]-out_po[n_freq][i-1][0]) - 
               ( in_po[n_freq][i+1]   -in_po[n_freq][i-1]),2.) < .10) {
          sum_coupler += in_po[n_freq][i] - out_po[n_freq][i][0];
          j++;
	}
      }
      if (j != 0) {
	out_cal[n_freq][0] = sum_coupler/j;}
      else {
	out_cal[n_freq][0] = 1000.;}
    }
    if (fl_get_button(fd_calibrate->CAL_2_path)) {
      switch_path_b(switch_driver, switch_driver_model);
      transfer_curve();

      for(n_freq=0; n_freq<max_freq; n_freq++) {
	for(i=1; i<drive_levels[n_freq]+1 ; i++) {
          in_po[n_freq][i] += in_cal[n_freq];
	}
      }

      for(n_freq=0; n_freq<max_freq; n_freq++) {
	sum_coupler = 0.; j = 0;
	for(i=2; i<drive_levels[n_freq] ; i++) {
          if(pow((out_po[n_freq][i+1][0]-out_po[n_freq][i-1][0]) - 
        	 ( in_po[n_freq][i+1]   -in_po[n_freq][i-1]),2.) < .10) {
            sum_coupler += in_po[n_freq][i] - out_po[n_freq][i][0];
            j++;
          }
	}
	if (j != 0) {
          out_cal[n_freq][1] = sum_coupler/j;}
	else {
          out_cal[n_freq][1] = 1000.;}
      }
    }
    for(n_freq=0; n_freq<max_freq; n_freq++) {
       printf("Cal = %f, %f, j = %d\n", out_cal[n_freq][0], out_cal[n_freq][1], n_freq);}
    fl_set_menu_item_mode(fd_calibrate->CAL_file_menu, 2, FL_PUP_NONE);
  }
}

/*
 *   write input and output calibrations to respective files
 */
void save_cal(int file)
{
  int n_freq;
  FILE *in_file, *out_file;
  
  switch (file) {
      case 1:
         if ((in_file = fopen(fl_get_input(fd_calibrate->CAL_in_cal),"w")) == NULL) {
           sprintf(message,"Couldn't open %s for writing\n", fl_get_input(fd_calibrate->CAL_in_cal));
           fl_show_message(message,"Try new input calibration","");}
         else {
           for(n_freq=0; n_freq<max_freq; n_freq++) {
             fprintf(in_file,"%f\t%f\n",freq[n_freq], in_cal[n_freq]);
           }
           (void) fclose(in_file);
         }
         break;
      case 2:
         if ((out_file = fopen(fl_get_input(fd_calibrate->CAL_out_cal),"w")) == NULL) {
           sprintf(message,"Couldn't open %s for writing\n", fl_get_input(fd_calibrate->CAL_out_cal));
           fl_show_message(message,"Try new output calibration","");}
         else {
           if (fl_get_button(fd_calibrate->CAL_2_path)) {
             for(n_freq=0; n_freq<max_freq; n_freq++) {
               fprintf(out_file,"%f\t%f\t%f\n",freq[n_freq], out_cal[n_freq][0], out_cal[n_freq][1]);
             }
           }
           else {
             for(n_freq=0; n_freq<max_freq; n_freq++) {
               fprintf(out_file,"%f\t%f\n",freq[n_freq], out_cal[n_freq][0]);
             }
           }
           (void) fclose(out_file);
          }
         break;
      default:
         return;
         break;
  }
}


