/*
 *    Program to drive RF sweeper and power meters to determine power
 *    and gain for an RF amplifier.  Part of e-factory
 *    Programmer:	Danny Holstein
 *    Started:		1/27/97
 */

#include "forms.h"
#include <stdlib.h>
#include "e_factory.h"
#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <ib.h>
#include "drivers.h"

#define USE_SRQ 1
#define READS 5
#define WAIT 1
#define MAX_RES 1024

extern int ibsta,ibcnt,iberr;
char cmd[130], message[1024];
char res[MAX_RES];

FD_main *fd_main;
FD_LSG *fd_LSG;
FD_freq_prompt *fd_freq_prompt;
FD_power_prompt *fd_power_prompt;

void fl_plot(int points, float x_min, float x_max, float y_min, float y_max,
            char *xlabel, char *ylabel, char *label,
            float xdata[], float ydata[]);
float rd_dc_supply_cur(int model);

float freq[MAX_FREQ_PTS], power[MAX_FREQ_PTS], sat_drive[MAX_FREQ_PTS];
float in_po[MAX_FREQ_PTS][MAX_DRIVE_PTS], out_po[MAX_FREQ_PTS][MAX_DRIVE_PTS][2];
float rf_drive_lo, rf_drive_hi, drive_pwr, set_power, start_sweep, stop_sweep;
float del_freq, f_start, f_stop, del_drive, drive_del_freq;
float mtr_range_drive[MAX_FREQ_PTS][6], mtr_range_op[6];
int mtr_min_range[MAX_FREQ_PTS], drive_levels[MAX_FREQ_PTS];
float drive_start[MAX_FREQ_PTS], drive_stop[MAX_FREQ_PTS], drive_pwr_lo, drive_pwr_hi;
float current[MAX_FREQ_PTS][MAX_DRIVE_PTS];

#include "util.h"

int read_cal();
int read_cplr_factors();

void LSG_start_cb(FL_OBJECT *ob, long data)
{
   int i, j, k, n, n_freq, last_output_range, min_range;
   int sweeper, in_meter, out_meter;
   int sweeper_model, in_meter_model, out_meter_model, mtr_level;
   int switch_driver, switch_driver_model, channel;
   int limit_drive;
   float probable_pwr, sat_power, sat_drive_pwr, drive;
   float cal_del_freq, cplr_del_freq, calibration;
   float a_range_lo, a_range_hi, b_range_lo, b_range_hi;

   SWEEPER_PAR *parameters;
/*
 *    start button has been pressed, load input values
 */
  f_start = atof(fl_get_input(fd_LSG->LSG_start_freq));
  f_stop = atof(fl_get_input(fd_LSG->LSG_stop_freq));
  max_freq =  fl_get_counter_value(fd_LSG->LSG_freqs);
  min_range =  fl_get_counter_value(fd_LSG->LSG_range);
  del_drive =  fl_get_counter_value(fd_LSG->LSG_drive_steps);
  a_range_lo = atof(fl_get_input(fd_LSG->LSG_a_start_freq));
  a_range_hi = atof(fl_get_input(fd_LSG->LSG_a_stop_freq));
  b_range_lo = atof(fl_get_input(fd_LSG->LSG_b_start_freq));
  b_range_hi = atof(fl_get_input(fd_LSG->LSG_b_stop_freq));
/*
 *   initialize basic variables
 */
  del_freq = 0.;
  if (max_freq > 1) {del_freq = (f_stop - f_start)/(max_freq - 1);}
  for(n_freq=0; n_freq<max_freq; n_freq++) {freq[n_freq] = f_start + n_freq*del_freq;}
  if (del_freq<0. || f_start==0. ) {
     for(i=1; i<=6; i++) {fl_set_menu_item_mode(fd_LSG->LSG_display_menu, i, FL_PUP_GRAY);}
     for(i=1; i<=2; i++) {fl_set_menu_item_mode(fd_LSG->LSG_file_menu, i, FL_PUP_GRAY);}
     fl_show_message("Check Frequencies",
                     "Re-enter data","");
     return;}
/*
 *   initialize sweeper, check band, read calibration and turn off RF for
 *   power meter zeroing
 */
  if((sweeper = ibfind("sweeper")) & ERR){
    gpiberr("ibfind err");
    exit(1);
  }
  if(ibclr(sweeper) & ERR){
    gpiberr("clr err");
    exit(1);
  }
  sweeper_model = atoi(parse("sweeper","type"));
  parameters = sweeper_init(sweeper, sweeper_model);
  rf_drive_lo = parameters->rf_drive_lo;
  rf_drive_hi = parameters->rf_drive_hi;
  start_sweep = parameters->start_sweep;
  stop_sweep = parameters->stop_sweep;
  mtr_level = parameters->mtr_level;
   
  if (start_sweep > f_start) {
    fl_show_message("Start frequency below sweeper minimum",
                    "Re-enter data","");
    return;}
  if (stop_sweep < f_stop) {
    fl_show_message("Stop frequency above sweeper maximum",
                    "Re-enter data","");
    return;}
      
  sweeper_rf_off(sweeper, sweeper_model);
  
  if (read_cal() != 0 || read_cplr_factors() != 0) return;
  if (in_cal_freq[0] > f_start || out_cal_freq[0] > f_start ||
      in_cplr_freq[0] > f_start || out_cplr_freq[0] > f_start) {
    fl_show_message("Start frequency below calibration minimum",
                    "Re-enter data","");
    return;}
  if (in_cal_freq[in_cal_pts-1] < f_stop || out_cal_freq[out_cal_pts-1] < f_stop ||
      in_cplr_freq[in_cplr_pts-1] < f_stop || out_cplr_freq[out_cplr_pts-1] < f_stop) {
    fl_show_message("Stop frequency above calibration maximum",
                    "Re-enter data","");
    return;}
  
  if (fl_check_forms() == fd_LSG->LSG_abort) return;
/*
 *   initialize and zero power meters
 */
  if (! fl_get_button(fd_LSG->DC_power)) {
    fl_show_message("de-energize DUT",
                    "zero meters","");
  }
  in_meter_model = atoi(parse("in_meter","type"));
  if((in_meter = ibfind("in_meter")) & ERR){
    gpiberr("ibfind err");
    exit(1);
  }
  if( ibtmo(in_meter,15) & ERR ){
    gpiberr("tmo err");
    exit(1);
  }
  mtr_zero(in_meter, in_meter_model);
  
  out_meter_model = atoi(parse("out_meter","type"));
  if((out_meter = ibfind("out_meter")) & ERR){
    gpiberr("ibfind err");
    exit(1);
  }
  if( ibtmo(out_meter,15) & ERR){
    gpiberr("tmo err");
    exit(1);
  }
  mtr_zero(out_meter, out_meter_model);
/*
 *   read status and check to see if meters have been zeroed
 */
  for (i=1; i<30; i++) {
    sleep(1);
    if (mtr_zeroed(in_meter, in_meter_model) == 1 &&
        mtr_zeroed(out_meter, out_meter_model) == 1) break;
    if (i>=25) exit(1);
  }
  if (fl_check_forms() == fd_LSG->LSG_abort) return;
  
  if (! fl_get_button(fd_LSG->DC_power)) {
    fl_show_message("energize DUT",
                    "RF performance","");
  }
/*
 *   read drive box, if applicable
 */
  limit_drive = (fl_get_button(fd_LSG->LSG_drive_limit_button) || 
                 fl_get_button(fd_LSG->LSG_drive_box_button));
  if (fl_get_button(fd_LSG->LSG_drive_limit_button)) {
    drive_start[0] = atof(fl_get_input(fd_LSG->LSG_drive_low));
    drive_stop[0] = atof(fl_get_input(fd_LSG->LSG_drive_high));
    for(n_freq=1; n_freq<max_freq; n_freq++) {
      drive_start[n_freq] = drive_start[0];
      drive_stop[n_freq] = drive_stop[0];}
  }
  if (fl_get_button(fd_LSG->LSG_drive_box_button)) {
    if (read_drives() != 0) return;
    drive_del_freq = in_drive_freqs[1] - in_drive_freqs[0];
    for(n_freq=0; n_freq<max_freq; n_freq++) {
      if (freq[n_freq] == in_drive_freqs[drive_pts-1]) {
	drive_start[n_freq] = in_drive_start[drive_pts-1];
	drive_stop[n_freq] = in_drive_stop[drive_pts-1];}
      else {
	n = (freq[n_freq] - in_drive_freqs[0])/drive_del_freq;
	drive_start[n_freq]  =  (in_drive_freqs[n+1] - freq[n_freq])/drive_del_freq * in_drive_start[n] 
                              + (freq[n_freq] - in_drive_freqs[n])/drive_del_freq * in_drive_start[n+1];
	drive_stop[n_freq]  =  (in_drive_freqs[n+1] - freq[n_freq])/drive_del_freq * in_drive_stop[n] 
                              + (freq[n_freq] - in_drive_freqs[n])/drive_del_freq * in_drive_stop[n+1];
      }
    }
  }
/*
 *   subtract calibration from drive limits to get correct, calibrated drive.
 */
  if (limit_drive) {
    cal_del_freq = in_cal_freq[1] - in_cal_freq[0];
    cplr_del_freq = in_cplr_freq[1] - in_cplr_freq[0];
    for(n_freq=0; n_freq<max_freq; n_freq++) {
      if (freq[n_freq] == in_cal_freq[in_cal_pts-1]) {
	calibration = in_cal[in_cal_pts-1]; }
      else {
	n = (freq[n_freq] - in_cal_freq[0])/cal_del_freq;
	calibration  =  (in_cal_freq[n+1] - freq[n_freq])/cal_del_freq * in_cal[n] 
                      + (freq[n_freq] - in_cal_freq[n])/cal_del_freq * in_cal[n+1];
      }

      if (freq[n_freq] == in_cplr_freq[in_cplr_pts-1]) {
	calibration -= 10*log10(in_cplr[in_cplr_pts-1]); }
      else {
	n = (freq[n_freq] - in_cplr_freq[0])/cplr_del_freq;
	calibration -= 10*log10((in_cplr_freq[n+1] - freq[n_freq])/cplr_del_freq * in_cplr[n] 
                        	+ (freq[n_freq] - in_cplr_freq[n])/cplr_del_freq * in_cplr[n+1]);
      }

      drive_start[n_freq] -= calibration;
      drive_stop[n_freq] -= calibration;
    }
  }
  
  for (i = 1; i <= 5; i++) {mtr_range_op[i] = -1000.;}
/*
 ******  Read Transfer Curves, main loop ***************************
 */
 
  for(n_freq=0; n_freq<max_freq; n_freq++) {
    
    if (fl_get_button(fd_LSG->LSG_2_path)) {
      if((switch_driver = ibfind("switch_driver")) & ERR){
        gpiberr("ibfind err");
        exit(1);
      }
      switch_driver_model = atoi(parse("switch_driver","type"));
      if (a_range_lo <= freq[n_freq] && freq[n_freq] <= a_range_hi) {
        switch_path_a(switch_driver, switch_driver_model);}
      if (b_range_lo <= freq[n_freq] && freq[n_freq] <= b_range_hi) {
        switch_path_b(switch_driver, switch_driver_model);}
    }
/*
 *   record power ranges for frequency
 */
    
    set_sweeper_on(sweeper, freq[n_freq], rf_drive_lo, sweeper_model);
  
    i = min_range;
    mtr_min_range[n_freq] = 5;
    if (mtr_level) {   /* The program will use meter range switching to extend the dynamic range */
      while (i<6) {
	mtr_range_drive[n_freq][i] = pwr_set_range(in_meter,i,in_meter_model);
	i++ ;
      }
      for (i=min_range; i<6; i++) {
	if ( mtr_range_drive[n_freq][i] != -1000.0 &
            (mtr_range_drive[n_freq][i+1] 
                       - mtr_range_drive[n_freq][i]) > 3.0 ) {
          if ( i < mtr_min_range[n_freq]) {mtr_min_range[n_freq] = i;}
          break;
	}
      }
    }
    else {   /* The program will use the sweeper dynamic range */
      if (rf_drive_lo - 4 < parameters->rf_drive_lo ) {set_power = parameters->rf_drive_lo;}
      else {set_power = rf_drive_lo - 4;}
      while (set_power<rf_drive_hi) {
        set_rf_power(sweeper,set_power,sweeper_model);
	mtr_range_drive[n_freq][i] = pwr_set_range(in_meter,i,in_meter_model);
	if (mtr_range_drive[n_freq][i] != -1000.0) {
	  rf_drive_lo = set_power + 2;
	  mtr_min_range[n_freq] = min_range;
          for (i=min_range+1; i<6; i++) {mtr_range_drive[n_freq][i] = 100.0;}
	  break; }
	set_power = set_power + 2.0 ;
      }
      if (mtr_range_drive[n_freq][i] == -1000.0) {
	  printf("no drive measured");
	  return; }
    }
  
/*
 *  "temporary" work-around  
 */
           if((out_meter = ibfind("out_meter")) & ERR){
             gpiberr("ibfind err");                    
             exit(1);
           }
    printf("frequency = %.3f\n",freq[n_freq]);
/*
 *   record transfer curve for frequency, use narrow range for drive box;
 *   use all available drive otherwise.
 */
   n = 0; drive_levels[n_freq] = n;
   last_output_range = 1;
   if (limit_drive) {
     drive_pwr_lo = drive_start[n_freq];
     drive_pwr_hi = drive_stop[n_freq]+.5;}
   else {
     drive_pwr_lo = mtr_range_drive[n_freq][mtr_min_range[n_freq]];
     drive_pwr_hi = 20;}
     
   for (drive_pwr=drive_pwr_lo; drive_pwr<drive_pwr_hi;  drive_pwr += del_drive) {
     for (i = 5; i >= mtr_min_range[n_freq]; i--) {
       if (drive_pwr >= mtr_range_drive[n_freq][i]) {
         set_power = drive_pwr - mtr_range_drive[n_freq][i] + rf_drive_lo ;
         if (set_power < rf_drive_lo ) {set_power = rf_drive_lo ;}
         if (set_power > rf_drive_hi) break;
         set_rf_power(sweeper, set_power, sweeper_model);
       
         if (mtr_level) {set_range_rd(in_meter, i,in_meter_model);}
	 else {if (n==0) {set_range_rd(in_meter, i,in_meter_model);}
	         else{set_range_rd(in_meter, read_range(in_meter, in_meter_model),in_meter_model);}}
         if (limit_drive) {
           for (k=1; k<5; k++) {
             drive = read_power(in_meter, in_meter_model);
             set_power = set_power - (drive - drive_pwr)*.75;
             set_rf_power(sweeper, set_power, sweeper_model);
             set_range_rd(in_meter, read_range(in_meter, in_meter_model), in_meter_model);
             if (fabs(drive - drive_pwr) < .2) break;
           }
         }
         if (n == 0) {set_range_rd(out_meter, 0, out_meter_model);}
         else {
           probable_pwr = out_po[n_freq][n-1][0] + drive_pwr - in_po[n_freq][n-1]; j = 0;
           for (i = 1; i < 5; i++) {
             if (mtr_range_op[i] != -1000.0 &&
                 probable_pwr > mtr_range_op[i] &&
                 mtr_range_op[i+1] != -1000.0 &&
                 probable_pwr < mtr_range_op[i+1]) { j = i; break;}
           }
           if (mtr_range_op[5] != -1000.0 &&
               probable_pwr > mtr_range_op[5]) { j = 5;}
           set_range_rd(out_meter, j, out_meter_model);
         }
       
         n++; drive_levels[n_freq] = n;
         if (n >=  MAX_DRIVE_PTS - 1) break;
         
         in_po[n_freq][n] = read_power(in_meter, in_meter_model);
         if (in_po[n_freq][n] == -1000.0) {
           fl_show_message("Input meter read error","Check connections","");
           return;
         }
       
         out_po[n_freq][n][0] = read_power(out_meter, out_meter_model);
         if (out_po[n_freq][n][0] == -1000.0) {
           fl_show_message("Output meter read error","Check connections","");
           return;
         }
       
         j = read_range(out_meter, out_meter_model);
         if (j > last_output_range ) {
           mtr_range_op[j] = out_po[n_freq][n][0];
           last_output_range = j;
         }
	 if (fl_get_button(fd_LSG->DC_power)) {
	   current[n_freq][n] = rd_dc_supply_cur(1);
	 }
      	 if (fl_check_forms() == fd_LSG->LSG_abort) return;
         printf("%d, %.2f, %.2f, %.2f\n",n, in_po[n_freq][n], out_po[n_freq][n][0],set_power);
         break;
       }
     }
     if (set_power > rf_drive_hi & i >= 5) break ;     /* input maximum */
     if (n > 1) {                                      /* test for output saturation */
       if ((out_po[n_freq][n][0] <=
            out_po[n_freq][n-1][0] - .05 && !(limit_drive) &&
             in_po[n_freq][n] >
             in_po[n_freq][n-1] + .5 ) ||
           n >=  MAX_DRIVE_PTS - 1) break ;
     }
   }
 }
/*
 ******  Read Transfer Curves, end loop ***************************
 */
 
/*
 *       alternative to all the transfer curves
 *   o   compute small signal gain across the band
 *   o   calculate saturated drive across band (assuming the power stays within 8 db)
 *   o   populate drive levels at 5 db steps in small signal, 2 db steps large signal
 *   o   curve fit transfer curves and determine saturated drive.
 */
 
/*
 *   resolve saturated power and drive to better than LSG resolution
 */
  for(n_freq=0; n_freq<max_freq; n_freq++) {
    if (fl_get_button(fd_LSG->LSG_2_path)) {
      if((switch_driver = ibfind("switch_driver")) & ERR){
        gpiberr("ibfind err");
        exit(1);
      }
      switch_driver_model = atoi(parse("switch_driver","type"));
      if (a_range_lo <= freq[n_freq] && freq[n_freq] <= a_range_hi) {
        switch_path_a(switch_driver, switch_driver_model);}
      if (b_range_lo <= freq[n_freq] && freq[n_freq] <= b_range_hi) {
        switch_path_b(switch_driver, switch_driver_model);}
    }

    del_drive =  fl_get_counter_value(fd_LSG->LSG_drive_steps);
    power[n_freq] = out_po[n_freq][drive_levels[n_freq]][0];
    sat_drive[n_freq] = in_po[n_freq][drive_levels[n_freq]];
    for(i=drive_levels[n_freq]-1; i>=2; i--) {
      if (out_po[n_freq][i][0] > power[n_freq]) {
        power[n_freq] = out_po[n_freq][i][0];
        sat_drive[n_freq] = in_po[n_freq][i];
      }
      if (out_po[n_freq][i][0] < out_po[n_freq][i+11][0]) {break;}
    }

    for(j=min_range; j<6; j++){
      if (mtr_range_op[j+1] < mtr_range_op[j]+2 &&
          mtr_range_op[j] != -1000) {break;}
    }
    if (limit_drive) {del_drive  = 0.0;}
    sat_drive_pwr = sat_drive[n_freq];
    while (del_drive > fl_get_counter_value(fd_LSG->LSG_resolution)) {
      for (drive_pwr = sat_drive_pwr-del_drive;
           drive_pwr <= sat_drive_pwr+del_drive;  drive_pwr += del_drive) {
	for (i = 5; i >= mtr_min_range[n_freq]; i--) {
          if (drive_pwr >= mtr_range_drive[n_freq][i]) {
            set_power = drive_pwr - mtr_range_drive[n_freq][i] + rf_drive_lo ;
            if (set_power < rf_drive_lo ) {set_power = rf_drive_lo ;}
            if (set_power > rf_drive_hi) break;
            set_sweeper_on(sweeper, freq[n_freq], set_power, sweeper_model);

            set_range_rd(in_meter, i, in_meter_model);
            set_range_rd(out_meter, j, out_meter_model);
            sat_power = read_power(out_meter, out_meter_model);
	    if (fl_get_button(fd_LSG->DC_power)) {
	      (void) rd_dc_supply_cur(1);
	    }
	    if (fl_check_forms() == fd_LSG->LSG_abort) return;
            printf("%.2f, %.2f, %.2f\n",freq[n_freq], drive_pwr, sat_power);
            if (sat_power > power[n_freq]) {
              power[n_freq] = sat_power;
              sat_drive[n_freq] = read_power(in_meter, in_meter_model);
              sat_drive_pwr = drive_pwr;
              printf("******** %.2f\n",sat_drive[n_freq]);
            }
            break;
          }
	}
      }
      del_drive = del_drive/2;
    }
  }
/*
 *   add output power calibration
 */
  channel = 0;
  cal_del_freq = out_cal_freq[1] - out_cal_freq[0];
  cplr_del_freq = out_cplr_freq[1] - out_cplr_freq[0];
  for(i=0; i<max_freq; i++) {
    if (fl_get_button(fd_LSG->LSG_2_path)) {   /* switch cals for appropriate filter */
      if (a_range_lo <= freq[i] && freq[i] <= a_range_hi) {
        channel = 0;}
      if (b_range_lo <= freq[i] && freq[i] <= b_range_hi) {
        channel = 1;}
    }
    if (freq[i] == out_cal_freq[out_cal_pts-1]) {
      calibration = out_cal[out_cal_pts-1][channel]; }
    else {
      n = (freq[i] - out_cal_freq[0])/cal_del_freq;
      calibration  =  (out_cal_freq[n+1] - freq[i])/cal_del_freq * out_cal[n][channel] 
                    + (freq[i] - out_cal_freq[n])/cal_del_freq * out_cal[n+1][channel];
    }
    
    if (freq[i] == out_cplr_freq[out_cplr_pts-1]) {
      calibration -= 10*log10(out_cplr[out_cplr_pts-1]); }
    else {
      n = (freq[i] - out_cplr_freq[0])/cplr_del_freq;
      calibration -= 10*log10((out_cplr_freq[n+1] - freq[i])/cplr_del_freq * out_cplr[n] 
                              + (freq[i] - out_cplr_freq[n])/cplr_del_freq * out_cplr[n+1]);
    }
    
    for(j=1; j<=drive_levels[i]; j++) {
      out_po[i][j][0] += calibration;
    }
    power[i] += calibration;
  }
/*
 *   add input drive calibration
 */
  cal_del_freq = in_cal_freq[1] - in_cal_freq[0];
  cplr_del_freq = in_cplr_freq[1] - in_cplr_freq[0];
  for(i=0; i<max_freq; i++) {
    if (freq[i] == in_cal_freq[in_cal_pts-1]) {
      calibration = in_cal[in_cal_pts-1]; }
    else {
      n = (freq[i] - in_cal_freq[0])/cal_del_freq;
      calibration  =  (in_cal_freq[n+1] - freq[i])/cal_del_freq * in_cal[n] 
                    + (freq[i] - in_cal_freq[n])/cal_del_freq * in_cal[n+1];
    }
    
    if (freq[i] == in_cplr_freq[in_cplr_pts-1]) {
      calibration -= 10*log10(in_cplr[in_cplr_pts-1]); }
    else {
      n = (freq[i] - in_cplr_freq[0])/cplr_del_freq;
      calibration -= 10*log10((in_cplr_freq[n+1] - freq[i])/cplr_del_freq * in_cplr[n] 
                              + (freq[i] - in_cplr_freq[n])/cplr_del_freq * in_cplr[n+1]);
    }
    
    for(j=1; j<=drive_levels[i]; j++) {
      in_po[i][j] += calibration;
    }
    sat_drive[i] += calibration;
  }
  
  for(i=1; i<=6; i++) {fl_set_menu_item_mode(fd_LSG->LSG_display_menu, i, FL_PUP_NONE);}
  for(i=1; i<=2; i++) {fl_set_menu_item_mode(fd_LSG->LSG_file_menu, i, FL_PUP_NONE);}
  
  sweeper_rf_off(sweeper, sweeper_model);
  if (fl_get_button(fd_LSG->DC_power)) {
    (void) rd_dc_supply_cur(1);
  }
}

 
void LSG_display_menu_cb(FL_OBJECT *ob, long data)
{
  int i, n;
  float x[127], y[127];
  float del_power;
  int sub_menu;
   switch (fl_get_menu(ob)) {
      case 1:   /* Display output power */
         fl_deactivate_form(fd_LSG->LSG);
         sub_menu = fl_defpup(FL_ObjWin(ob),"Decibels|Watts");
         switch((int) fl_dopup(sub_menu))
         {
            case 1:   /* Decibels */
              for (i=0 ; i<max_freq ; i++) {
                x[i] = freq[i]; y[i] = power[i];
              }
              fl_plot(max_freq, freq[0], freq[max_freq-1],
                      0., 60.0, "Frequency (GHz)",
                     "Output Power(dbm)", fl_get_input(fd_main->Part), x, y);
              break;
            case 2:   /* Watts */
              for (i=0 ; i<max_freq ; i++) {
                x[i] = freq[i]; y[i] = pow(10.0, power[i]/10.0 - 3.0);
              }
              fl_plot(max_freq, freq[0], freq[max_freq-1],
                      0., 100.0, "Frequency (GHz)",
                     "Output Power(watts)", fl_get_input(fd_main->Part), x, y);
              break;
         }

         fl_activate_form(fd_LSG->LSG);
         break;
      case 2:   /* Display Saturated Gain (db) */
         for (i=0 ; i<max_freq ; i++) {
           x[i] = freq[i]; y[i] = power[i] - sat_drive[i];
         }
         fl_deactivate_form(fd_LSG->LSG);
         fl_plot(max_freq, freq[0], freq[max_freq-1],
                 0., 60.0, "Frequency (GHz)",
                "Saturated Gain (db)", fl_get_input(fd_main->Part),  x, y);
         fl_activate_form(fd_LSG->LSG);
         break;
      case 3:   /* Display Saturated Drive (dbm) */
         for (i=0 ; i<max_freq ; i++) {
           x[i] = freq[i]; y[i] = sat_drive[i];
         }
         fl_deactivate_form(fd_LSG->LSG);
         fl_plot(max_freq, freq[0], freq[max_freq-1],
                 -20., 50.0, "Frequency (GHz)",
                "Saturated Drive (dbm)", fl_get_input(fd_main->Part),  x, y);
         fl_activate_form(fd_LSG->LSG);
         break;
      case 4:   /* Display Small Signal Gain (db) */
         for (i=0 ; i<max_freq ; i++) {
           x[i] = freq[i]; y[i] = out_po[i][1][0] - in_po[i][1];
         }
         fl_deactivate_form(fd_LSG->LSG);
         fl_plot(max_freq, freq[0], freq[max_freq-1],
                 0., 60.0, "Frequency (GHz)",
                "Small Signal Gain (db)", fl_get_input(fd_main->Part),  x, y);
         fl_activate_form(fd_LSG->LSG);
         break;
      case 5:   /* Display transfer curve for one frequency */
         n = max_freq/2;
         fl_deactivate_form(fd_LSG->LSG);
         if (max_freq ==1) {n = 0;}
         else {
           fd_freq_prompt = create_form_freq_prompt();
           fl_set_counter_bounds(fd_freq_prompt->TRANSFER_freq,freq[0],freq[max_freq-1]);
           fl_set_counter_step(fd_freq_prompt->TRANSFER_freq, del_freq, del_freq);
           fl_set_counter_precision(fd_freq_prompt->TRANSFER_freq,2);
           fl_set_counter_value(fd_freq_prompt->TRANSFER_freq, freq[n]);
           fl_show_form(fd_freq_prompt->freq_prompt,FL_PLACE_CENTER,FL_FULLBORDER,"Transfer Curve");
           while (fl_do_forms() != fd_freq_prompt->TRANSFER_continue) ;
           n = (fl_get_counter_value(fd_freq_prompt->TRANSFER_freq) - freq[0]) / del_freq + .5;
           fl_hide_form(fd_freq_prompt->freq_prompt);
           fl_free_form(fd_freq_prompt->freq_prompt);
         }
 
         for (i=1 ; i<=drive_levels[n] ; i++) {
           x[i-1] = in_po[n][i]; y[i-1] = out_po[n][i][0];
         }
         sprintf(message,"%s @%.3f GHz", fl_get_input(fd_main->Part), freq[n]);
         fl_plot(drive_levels[n], x[0], x[drive_levels[n]-1],
                 y[0], y[drive_levels[n]-1], "Input Power (dbm)",
                "Output Power(dbm)", message,  x, y);
         fl_activate_form(fd_LSG->LSG);
         break;
      case 6:
         n = 0;
         fl_deactivate_form(fd_LSG->LSG);
         del_power = in_po[0][2]-in_po[0][1];
         
         fd_power_prompt = create_form_power_prompt();
         fl_set_counter_bounds(fd_power_prompt->POWER_drive,
                               0.0, in_po[0][drive_levels[0]]-in_po[0][1]);
         fl_set_counter_step(fd_power_prompt->POWER_drive,
                               del_power, del_power);
         fl_set_counter_precision(fd_power_prompt->POWER_drive,1);
         fl_set_counter_value(fd_power_prompt->POWER_drive, 0.0);
         fl_show_form(fd_power_prompt->power_prompt,FL_PLACE_CENTER,FL_FULLBORDER,"drive box power");
         while (fl_do_forms() != fd_power_prompt->POWER_continue) ;
         n = fl_get_counter_value(fd_power_prompt->POWER_drive) / del_power + 1.5;
         fl_hide_form(fd_power_prompt->power_prompt);
         fl_free_form(fd_power_prompt->power_prompt);

         sub_menu = fl_defpup(FL_ObjWin(ob),"Decibels|Watts");
         switch((int) fl_dopup(sub_menu))
         {
            case 1:
              for (i=0 ; i<max_freq ; i++) {
                x[i] = freq[i]; y[i] = out_po[i][n][0];
              }
              fl_plot(max_freq, freq[0], freq[max_freq-1],
                      0., 60.0, "Frequency (GHz)",
                     "Output Power(dbm)", fl_get_input(fd_main->Part), x, y);
              break;
            case 2:
              for (i=0 ; i<max_freq ; i++) {
                x[i] = freq[i]; y[i] = pow(10.0, out_po[i][n][0]/10.0 - 3.0);
              }
              fl_plot(max_freq, freq[0], freq[max_freq-1],
                      0., 100.0, "Frequency (GHz)",
                     "Output Power(watts)", fl_get_input(fd_main->Part), x, y);
              break;
         }
         fl_activate_form(fd_LSG->LSG);
         break;
      default:
         break;
   }
}


int LSG_check_exit()
{
   int i, n_freq;
   const char *fname;
   FILE *out_file;
   switch (fl_get_menu(fd_LSG->LSG_file_menu)) {
      case 1:  /*  Write sat. power to output file */
         fname = fl_show_file_selector("File To Save","","*.*","");
         if (fname == NULL) return 0;
         if ((out_file = fopen(fname,"w")) == NULL) {
           sprintf(message,"Couldn't open %s for writing\n", fname);
           fl_show_message(message,"","");}
         else {
           fprintf(out_file,"%s\t%s\t%s\n","Frequency", "Input Power (dbm)", "Output Power (dbm)");
           for(n_freq=0; n_freq<max_freq; n_freq++) {
               fprintf(out_file,"%f\t%f\t%f",freq[n_freq], sat_drive[n_freq], power[n_freq]);
               fprintf(out_file,"\n");
           }
           (void) fclose(out_file);
         }
         return 0;
         break;
      case 2:  /*  Write transfer curves to output file */
         fname = fl_show_file_selector("File To Save","","*.*","");
         if (fname == NULL) return 0;
         if ((out_file = fopen(fname,"w")) == NULL) {
           sprintf(message,"Couldn't open %s for writing\n", fname);
           fl_show_message(message,"","");}
         else {
           fprintf(out_file,"%s\t%s\t%s\n","Frequency", "Input Power (dbm)", "Output Power (dbm)");
           for(n_freq=0; n_freq<max_freq; n_freq++) {
             for(i=1; i<=drive_levels[n_freq]; i++) {
               fprintf(out_file,"%f\t%f\t%f",freq[n_freq], in_po[n_freq][i], out_po[n_freq][i][0]);
	       if (fl_get_button(fd_LSG->DC_power)) {
                 fprintf(out_file,"\t%f",current[n_freq][i]);
	       }
               fprintf(out_file,"\n");
             }
           fprintf(out_file,"\n");   /* blank line between frequencies */
           }
           (void) fclose(out_file);
         }
         return 0;
         break;
      case 3:   /* Exit */
         return 1;
         break;
      default:
         return 0;
         break;
   }
}

int
preemptive_handler(FL_OBJECT *ob, int event,
                          FL_Coord mx, FL_Coord my, int key, void *xev)
{
  const char *fname;

    switch(event)
    {
      case FL_DBLCLICK:
        fname = fl_show_file_selector("File To Load","","*.cal","");
        if (fname == NULL) return 0;
        fl_set_input(ob, fname);
        break;
      default:
        break;
    }
    return 0;
}

int
preemptive_handler_drive(FL_OBJECT *ob, int event,
                          FL_Coord mx, FL_Coord my, int key, void *xev)
{
  const char *fname;

    switch(event)
    {
      case FL_DBLCLICK:
        fname = fl_show_file_selector("File To Load","","*.rfd","");
        if (fname == NULL) return 0;
        fl_set_input(ob, fname);
        break;
      default:
        break;
    }
    return 0;
}

void LSG_help_menu_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void LSG_drive_limit_button_cb(FL_OBJECT *ob, long data)
{
  int released=0;
  fl_set_button(fd_LSG->LSG_drive_box_button, released);
}

void LSG_drive_box_button_cb(FL_OBJECT *ob, long data)
{
  int released=0;
  fl_set_button(fd_LSG->LSG_drive_limit_button, released);
}

/*---------------------------------------*/
FD_freq_prompt *create_form_freq_prompt(void)
{
  FL_OBJECT *obj;
  FD_freq_prompt *fdui = (FD_freq_prompt *) fl_calloc(1, sizeof(*fdui));

  fdui->freq_prompt = fl_bgn_form(FL_NO_BOX, 270, 130);
  obj = fl_add_box(FL_UP_BOX,0,0,270,130,"");
  obj = fl_add_text(FL_NORMAL_TEXT,10,10,250,40,"Choose frequency for transfer curve plot");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
  fdui->TRANSFER_freq = obj = fl_add_counter(FL_SIMPLE_COUNTER,60,50,140,30,"");
  fdui->TRANSFER_continue = obj = fl_add_button(FL_NORMAL_BUTTON,60,90,140,30,"Continue");
  fl_end_form();

  return fdui;
}
/*---------------------------------------*/

FD_power_prompt *create_form_power_prompt(void)
{
  FL_OBJECT *obj;
  FD_power_prompt *fdui = (FD_power_prompt *) fl_calloc(1, sizeof(*fdui));

  fdui->power_prompt = fl_bgn_form(FL_NO_BOX, 270, 130);
  obj = fl_add_box(FL_UP_BOX,0,0,270,130,"");
  obj = fl_add_text(FL_NORMAL_TEXT,10,10,250,40,"Choose drive for power plot\n(db over nominal)");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
  fdui->POWER_drive = obj = fl_add_counter(FL_SIMPLE_COUNTER,60,50,140,30,"");
  fdui->POWER_continue = obj = fl_add_button(FL_NORMAL_BUTTON,60,90,140,30,"Continue");
  fl_end_form();

  return fdui;
}
/*---------------------------------------*/

FD_LSG *create_form_LSG(void)
{
  int i;
  FL_OBJECT *obj;
  FD_LSG *fdui = (FD_LSG *) fl_calloc(1, sizeof(*fdui));

  fdui->LSG = fl_bgn_form(FL_NO_BOX, 380, 450);
  obj = fl_add_box(FL_UP_BOX,0,0,380,450,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,10,50,230,120,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,10,180,360,80,"");
  obj = fl_add_box(FL_UP_BOX,0,0,380,40,"");
  fdui->LSG_file_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,10,10,60,20,"File");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_menu(obj, "Save Power|Save Transfer|Exit");
    for(i=1; i<=2; i++) {fl_set_menu_item_mode(obj, i, FL_PUP_GRAY);}
  fdui->LSG_display_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,90,10,60,20,"Display");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_menu(obj,
    "Power ...|Large Sig. Gain|Saturated Drive|Small Sig. Gain|Transfer Curves|Power at Drive");
    for(i=1; i<=6; i++) {fl_set_menu_item_mode(obj, i, FL_PUP_GRAY);}
    fl_set_object_callback(obj,LSG_display_menu_cb,0);
  fdui->LSG_help_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,310,10,60,20,"Help");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,LSG_help_menu_cb,0);
  fdui->LSG_start_freq = obj = fl_add_input(FL_FLOAT_INPUT,100,60,80,30,"Start Frequency");
  fdui->LSG_stop_freq = obj = fl_add_input(FL_FLOAT_INPUT,100,90,80,30,"Stop Frequency");
  fdui->LSG_drive_box = obj = fl_add_input(FL_NORMAL_INPUT,110,220,110,30,"");
    fl_set_object_prehandler(obj, preemptive_handler_drive);
  fdui->LSG_drive_low = obj = fl_add_input(FL_FLOAT_INPUT,110,190,50,30,"");
  fdui->LSG_drive_high = obj = fl_add_input(FL_FLOAT_INPUT,170,190,50,30,"");
  fdui->LSG_start = obj = fl_add_button(FL_NORMAL_BUTTON,260,60,110,30,"Start");
    fl_set_object_color(obj,FL_SPRINGGREEN,FL_PALEGREEN);
    fl_set_object_lcol(obj,FL_DARKTOMATO);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,LSG_start_cb,0);
  fdui->LSG_abort = obj = fl_add_button(FL_NORMAL_BUTTON,260,100,110,30,"Abort");
    fl_set_object_color(obj,FL_RED,FL_DARKTOMATO);
    fl_set_object_lcol(obj,FL_YELLOW);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
  fdui->LSG_drive_limit_button = obj = fl_add_checkbutton(FL_PUSH_BUTTON,10,190,30,30,"Drive Limits:");
    fl_set_object_callback(obj,LSG_drive_limit_button_cb,0);
  fdui->LSG_drive_box_button = obj = fl_add_checkbutton(FL_PUSH_BUTTON,10,220,30,30,"Drive Box File");
    fl_set_object_callback(obj,LSG_drive_box_button_cb,0);
  fdui->DC_power = obj = fl_add_lightbutton(FL_PUSH_BUTTON,260,140,110,30,"Device Power");
    fl_set_object_color(obj,FL_COL1,FL_YELLOW);
    fl_set_object_callback(obj,DC_power_cb,0);
  obj = fl_add_frame(FL_ENGRAVED_FRAME,10,270,360,80,"");
  fdui->LSG_in_cal = obj = fl_add_input(FL_NORMAL_INPUT,120,280,220,30,"Input Calibration File");
    fl_set_object_prehandler(obj, preemptive_handler);
  fdui->LSG_out_cal = obj = fl_add_input(FL_NORMAL_INPUT,120,310,220,30,"Output Calibration File");
    fl_set_object_prehandler(obj, preemptive_handler);
  fdui->LSG_freqs = obj = fl_add_counter(FL_NORMAL_COUNTER,70,130,160,30,"Frequencies");
    fl_set_counter_bounds(obj,1.0,MAX_FREQ_PTS);
    fl_set_counter_step(obj,1.0,10.0);
    fl_set_counter_precision(obj,0);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
  fdui->LSG_range = obj = fl_add_counter(FL_SIMPLE_COUNTER,290,230,70,20,"Min. Range");
    fl_set_counter_bounds(obj,1.0,5);
    fl_set_counter_step(obj,1.0,1.0);
    fl_set_counter_precision(obj,0);
    fl_set_counter_value(obj, 2.0);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
  fdui->LSG_resolution = obj = fl_add_counter(FL_SIMPLE_COUNTER,290,210,70,20,"Drive Res.");
    fl_set_counter_bounds(obj,.2,2.0);
    fl_set_counter_step(obj,.5,.5);
    fl_set_counter_precision(obj,1);
    fl_set_counter_value(obj, 1.2);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
  fdui->LSG_drive_steps = obj = fl_add_counter(FL_SIMPLE_COUNTER,290,190,70,20,"Drive Steps");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
    fl_set_counter_bounds(obj,1.0,5.0);
    fl_set_counter_step(obj,1.0,1.0);
    fl_set_counter_precision(obj,1);
    fl_set_counter_value(obj, 2.0);
  obj = fl_add_frame(FL_ENGRAVED_FRAME,10,360,360,80,"");
  fdui->LSG_a_start_freq = obj = fl_add_input(FL_FLOAT_INPUT,80,370,70,30,"Chan. A Start");
  fdui->LSG_a_stop_freq = obj = fl_add_input(FL_FLOAT_INPUT,80,400,70,30,"Chan. A Stop");
  fdui->LSG_b_start_freq = obj = fl_add_input(FL_FLOAT_INPUT,220,370,70,30,"Chan. B Start");
  fdui->LSG_b_stop_freq = obj = fl_add_input(FL_FLOAT_INPUT,220,400,70,30,"Chan. B Stop");
  fdui->LSG_2_path = obj = fl_add_checkbutton(FL_PUSH_BUTTON,320,380,20,20,"2 path (filters)");
    fl_set_object_lalign(obj,FL_ALIGN_BOTTOM);
  fl_end_form();

  return fdui;
}

/*
 * This is a simple error handling function
 *
 */
gpiberr(char *msg) {

printf("%s\n",msg);

printf("ibsta=0x%x  <",ibsta);

if ( ibsta & ERR )  printf(" ERR");
if ( ibsta & TIMO ) printf(" TIMO");
if ( ibsta & END )  printf(" END");
if ( ibsta & SRQI ) printf(" SRQI");
/*if ( ibsta & RQS )  printf(" RQS");*/
if ( ibsta & CMPL ) printf(" CMPL");
/*if ( ibsta & LOK )  printf(" LOK");*/
/*if ( ibsta & REM )  printf(" REM");*/
if ( ibsta & CIC )  printf(" CIC");
if ( ibsta & ATN )  printf(" ATM");
if ( ibsta & TACS ) printf(" TACS");
if ( ibsta & LACS ) printf(" LACS");
/*if ( ibsta & DTAS ) printf(" DATS");*/
/*if ( ibsta & DCAS ) printf(" DCTS");*/

printf(" >\n");

printf("iberr= %d", iberr);
if ( iberr == EDVR) printf(" EDVR <OS Error>\n");
if ( iberr == ECIC) printf(" ECIC <Not CIC>\n");
if ( iberr == ENOL) printf(" ENOL <No Listener>\n");
if ( iberr == EADR) printf(" EADR <Adress Error>\n");
if ( iberr == EARG) printf(" ECIC <Invalid Argument>\n");
if ( iberr == ESAC) printf(" ESAC <No Sys Ctrlr>\n");
if ( iberr == EABO) printf(" EABO <Operation Aborted>\n");
if ( iberr == ENEB) printf(" ENEB <No Gpib Board>\n");
if ( iberr == EOIP) printf(" EOIP <Async I/O in prg>\n");
if ( iberr == ECAP) printf(" ECAP <No Capability>\n");
if ( iberr == EFSO) printf(" EFSO <File sys. error>\n");
if ( iberr == EBUS) printf(" EBUS <Command error>\n");
if ( iberr == ESTB) printf(" ESTB <Status byte lost>\n");
if ( iberr == ESRQ) printf(" ESRQ <SRQ stuck on>\n");
if ( iberr == ETAB) printf(" ETAB <Table Overflow>\n");
if ( iberr == EPAR) printf(" EPAR <Parse Error in Config>\n");
if ( iberr == ECFG) printf(" ECFG <Can't open Config>\n");
if ( iberr == ETAB) printf(" ETAB <Device Table Overflow>\n");
if ( iberr == ENSD) printf(" ENSD <Configuration Error>\n");


printf("ibcnt= %d\n", ibcnt );
printf("\n");


/*ibonl(0);*/
}

